-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

---------------------
-- Prep the module --
---------------------
local M = {}
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 11000

-----------------
-- Debug stuff --
-----------------
local ugfxCalled = 0
local breakGroupDumped = 0

---------------------------
-- Tuning menu variables --
---------------------------

-- Air brake momentary/toggle
local airBrakeToggle = 0

-- Hover momentary/toggle
local hoverToggle = 1

---------------------
-- State variables --
---------------------

-- Last trigger state of air brake and hover
local lastTrigAirBrake = 0
local lastTrigHover = 0
local lastParkBrake = 0

-- Fire nodes
-- {["FireNodeName"] = {"BreakGroupName", FireNodeIndex}}
local fireNodesOfInterest = {
	{["wl5b"]	= {"wingouter_L", 2}},
	{["wl4bb"]	= {"wingouter_L", 3}},
	{["wl3b"]	= {"wingouter_L", 4}},
	{["wl4b"]	= {"wingouter_L", 5}},
	
	{["wl1b"]	= {"wingL0", 2}},
	{["wl0b"]	= {"wingL0", 3}},
	{["wmrb"]	= {"wingL0", 4}},
	{["wmlb"]	= {"wingL0", 5}},
	
	{["wr1b"]	= {"wingR0", 2}},
	{["wr0b"]	= {"wingR0", 3}},
	{["wmlb"]	= {"wingR0", 4}},
	{["wmrb"]	= {"wingR0", 5}},
	
	{["wr5b"]	= {"wingouter_R", 2}},
	{["wr4bb"]	= {"wingouter_R", 3}},
	{["wr3b"]	= {"wingouter_R", 4}},
	{["wr4b"]	= {"wingouter_R", 5}}
}

-- For broken break groups
-- {["BreakGroupName"] = {GroupBroken, FireNode1Id, FireNode2Id, FireNode3Id, FireNode4Id, ExplosionDuration}}
local brokenBeamsOfInterest = {
	{["wingouter_R"]	= {false, -1, -1, -1, -1, 0.5}},
	{["wingouter_L"]	= {false, -1, -1, -1, -1, 0.5}},
	{["wingR0"]			= {false, -1, -1, -1, -1, 0.5}},
	{["wingL0"]			= {false, -1, -1, -1, -1, 0.5}}
}

-- Beam fire timing
local beamFireCapable = 1
local beamFireTimeElapsed = 0
local beamFireTimeLimit = 1.0/30.0



------------------------------------
-- Determine edges on triggers    --
------------------------------------

-- Edge events
-- 0 = No edge
-- 1 = Rising edge
-- 2 = Falling edge

local function getTriggerEdges()
	local	edgeDirAB = 0
	local	edgeDirH = 0
	local	edgeDirPB = 0
	
	-- Trigger activated
	local airBrakeTrigger = (electrics.values['stolAirBrakes'] or 0)
	local hoverTrigger = (electrics.values['stolHoverOn'] or 0)
	local parkBrakeTrigger = (electrics.values['brake_combined'] or 0)
	
	-- Detect air brake edge
	if airBrakeTrigger == 1 then	if lastTrigAirBrake == 0 then edgeDirAB = 1 end
	else							if lastTrigAirBrake == 1 then edgeDirAB = 2 end end
	
	-- Detect hover edge
	if hoverTrigger == 1 then	if lastTrigHover == 0 then edgeDirH = 1 end
	else						if lastTrigHover == 1 then edgeDirH = 2 end end
	
	-- Detect parking brake edge
	if parkBrakeTrigger == 1 then	if lastParkBrake == 0 then edgeDirPB = 1 end
	else							if lastParkBrake == 1 then edgeDirPB = 2 end end
	
	-- New trigger states have become old trigger states
	lastTrigAirBrake = airBrakeTrigger
	lastTrigHover = hoverTrigger
	lastParkBrake = parkBrakeTrigger
	
	-- Return result
	return edgeDirAB, edgeDirH, edgeDirPB
end



-----------------------------------
-- Update hover and braking jets --
-----------------------------------
local function updateJets()
	-- Detect trigger edges
	-- Retrieve some electrics
	local edgeDirAB
	local edgeDirH
	local edgeDirPB
	local airBrakeON	= (electrics.values['airBrakeON']	or 0)
	local hoverON		= (electrics.values['hoverON']		or 0)
	edgeDirAB, edgeDirH, edgeDirPB = getTriggerEdges()
	
	---------------
	-- Air Brake --
	---------------
	
	-- Air brake is momentary
	-- Rising edge detected  --> Air brake ON
	-- Falling edge detected --> Air brake OFF
	if airBrakeToggle == 0 then
		if edgeDirAB == 1 then
			electrics.values['airBrakeON'] = 1
		elseif edgeDirAB == 2 then
			electrics.values['airBrakeON'] = 0
		end
		
	-- Air brake is toggled on rising edge
	elseif edgeDirAB == 1 then
		if airBrakeON == 0 then
			electrics.values['airBrakeON'] = 1
		else
			electrics.values['airBrakeON'] = 0
		end
	end
--	if electrics.values['airBrakeON'] == 1 then
--		electrics.values['abs'] = 1
--	else
--		electrics.values['abs'] = 0
--	end
		
		
	-----------
	-- Hover --
	-----------
	
	-- Hover is momentary
	-- Rising edge detected  --> Hover ON
	-- Falling edge detected --> Hover OFF
	if hoverToggle == 0 then
		if edgeDirH == 1 then
			electrics.values['hoverON'] = 1
			electrics.light_flash_highbeams(true)
		elseif edgeDirH == 2 then
			electrics.values['hoverON'] = 0
			electrics.light_flash_highbeams(false)
		end
		
	-- Hover is toggled on rising edge
	elseif edgeDirH == 1 then
		if hoverON == 0 then
			electrics.values['hoverON'] = 1
			electrics.light_flash_highbeams(true)
		else
			electrics.values['hoverON'] = 0
			electrics.light_flash_highbeams(false)
		end
	end
	
	------------------------------------
	-- BONUS: Parking brake indicator --
	------------------------------------
--	if edgeDirPB == 1 then
--		input.event("parkingbrake", 1, 0)
--	elseif edgeDirPB == 2 then
--		input.event("parkingbrake", 0, 0)
--	end
	if electrics.values.brake > 0.5 then
--		input.event("parkingbrake", 1, 0)
		electrics.values['abs'] = 1
	else
--		input.event("parkingbrake", 0, 0)
		electrics.values['abs'] = 0
	end
end



-----------------------
-- Update beam fires --
-----------------------
--local function updateBeamFires(dt)
--	
--	-- Capable of igniting beams
--	if beamFireCapable == 1 then
--		
--		-- Time to update beam fires
--		beamFireTimeElapsed = beamFireTimeElapsed + dt
--		if beamFireTimeElapsed >= beamFireTimeLimit then
--			beamFireTimeElapsed = 0
--			
--			-- Loop through all beam break groups of interest
--			for i, beam in pairs(brokenBeamsOfInterest) do
--				
--				-- Emit flames and smoke for beams on fire
--				if beam[1] == true then
--					
--					-- Still time for explosion flames and smoke
--					if beam[6] >= 0 then
--						brokenBeamsOfInterest[i][6] = beam[6] - dt
--						
--						
--						
--					-- Time for steady-state flames and smoke
--					else
--						
--					end
--				end
--			end
--		end
--	end
--end



--------------------------------------
-- Set fires for broken beam groups --
--------------------------------------
--local function setBeamFires(breakGroupNames)
--	
--	-- Loop through all of this beam's break groups
--	for i, v in pairs(breakGroupNames) do
--		
--		-- Get flaming status for this break group, if we're interested in it
--		local beamFlaming = brokenBeamsOfInterest[v]
--		if beamFlaming ~= nil then
--			
--			-- Set beam group on fire
--			brokenBeamsOfInterest[v][1] = true
--			
--			-- TODO: Emit explosion sound
--			
--			obj:addParticleByNodesRelative(	-- Add explosion flame particle
--				beamFlaming[2],				-- Explosion flame FROM node ID
--				beamFlaming[3],				-- Explosion flame TO node ID
--				5,							-- Explosion flame speed
--				29,							-- Explosion flame particle ID
--				3,							-- Explosion flame width
--				20)							-- Explosion flame count
--			obj:addParticleByNodesRelative(	-- Add explosion smoke particle
--				beamFlaming[2],				-- Explosion smoke FROM node ID
--				beamFlaming[3],				-- Explosion smoke TO node ID
--				10,							-- Explosion smoke speed
--				32,							-- Explosion smoke particle ID
--				3,							-- Explosion smoke width
--				2)							-- Explosion smoke count
--		end
--	end
--end



---------------------
-- Beam got broken --
---------------------
--local function beamBroken(id, energy)
--	
--	-- Get freshly broken beam's ID
--	local thisBrokenBeam = v.data.beams[id]
--	if thisBrokenBeam ~= nil then
--		
--		-- Get broken beam break group type
--		local thisBreakType = thisBrokenBeam.breakGroupType
--		
--		-- Only do stuff if the group will break with this beam
--		if thisBreakType == 0 then
--			
--			-- Get broken beam break group name(s)
--			local thisBreakGroup = thisBrokenBeam.breakGroup
--			if thisBreakGroup ~= nil then
--				if string.len(thisBreakGroup) > 0 then
--					
--					-- Sometimes beams are members of multiple groups, which will be in a table.
--					-- If only a member of one group, it's just a string. Put the string in a table.
--					local breakGroupNames = nil
--					if type(thisBreakGroup) == "string" then
--						breakGroupNames = {thisBreakGroup}
--					elseif type(thisBreakGroup) == "table" then
--						breakGroupNames = thisBreakGroup
--					end
--					
--					-- Set broken beam interfaces on fire
--					setBeamFires(breakGroupNames)
--				end
--			end
--		end
--	end
--end



---------------------------------------------
-- Graphics update function                --
-- Updates other stuff too according to dt --
---------------------------------------------
local function updateGFX(dt)
	
	-- Update hover and braking jets
	updateJets()
	
	-- Update beam fires
--	updateBeamFires()
	
	-- Debug message
	if ugfxCalled == 0 then
		ugfxCalled = 1
		print("AN12 STOL updateGFX called")
	end
end



---------------------------
-- Reset what we need to --
---------------------------
local function reset()
	lastTrigAirBrake = 0
	lastTrigHover = 0
	electrics.values['airBrakeON'] = 0
	electrics.values['hoverON'] = 0
	
--	for i, v in pairs(brokenBeamsOfInterest)
--		brokenBeamsOfInterest[i][1] = false
--		brokenBeamsOfInterest[i][6] = 0.5
--	end
--	beamFireTimeElapsed = 0
	
	print("AN12 STOL reset called")
end



-----------------------
-- Get fire node IDs --
-----------------------
--local function getFireNodeIDs(jbeamdata)
--	
--	-- Get IDs of all fire nodes
--	for i, node in pairs(v.data.nodes)
--		brokenBeamsOfInterest[fireNodesOfInterest[node.name][1]][fireNodesOfInterest[node.name][2]] = i
--	end
--	
--	-- Verify all fire node IDs found
--	for i, beam in pairs(brokenBeamsOfInterest)
--		for j = 2, 5 do
--			if beam[j] < 0 then
--				beamFireCapable = 0
--				break
--			end
--		end
--		if beamFireCapable == 0 then break end
--	end
--end



---------------------------
-- Initialize everything --
---------------------------
local function init(jbeamdata)
	
	-- Get toggle settings
	hoverToggle = (v.data.variables["$hb9k_hoverToggle"].val or 1)
	airBrakeToggle = (v.data.variables["$hb9k_brakeToggle"].val or 0)
	
	-- Get IDs of fire nodes
--	getFireNodeIDs(jbeamdata)
	
	-- Debug msg
	print("AN12 STOL init called")
end



----------------------
-- Public interface --
----------------------
M.init = init
M.reset = reset
M.updateGFX = updateGFX
--M.beamBroken = beamBroken
return M
